﻿using System.Drawing.Drawing2D;
using AirplaneWithRadar.Entities;
using AirplaneWithRadar.MovementStrategy;

namespace AirplaneWithRadar.PaintObjects
{
    public class PaintAirplane
    {
        public AirplaneEntity? AirplaneEntity { get; protected set; }
        private int pictWidth;
        private int pictHeight;
        protected int startPosX;
        protected int startPosY;
        protected readonly int planeWidth = 140;
        protected readonly int planeHeight = 60;

        public int GetPosX => startPosX;
        public int GetPosY => startPosY;
        public int GetWidth => planeWidth;
        public int GetHeight => planeHeight;

        public IMoveableObject GetMoveableObject => new PaintObjectAirplane(this);

        public bool CanMove(Movement dir)
        {
            if (AirplaneEntity == null)
            {
                return false;
            }
            return dir switch
            {
                //влево
                Movement.Left => startPosX - AirplaneEntity.Step > 0,
                //вверх
                Movement.Up => startPosY - AirplaneEntity.Step > 0,
                // вправо
                Movement.Right => startPosX + planeWidth + AirplaneEntity.Step < pictWidth,
                //вниз
                Movement.Down => startPosY + planeHeight + AirplaneEntity.Step < pictHeight,
                _ => false,
            };

        }

        public PaintAirplane(int speed, double weight, Color bodyColor, int width, int height)
        {
            if (planeWidth > width || planeHeight > height) { return; }

            pictHeight = height;
            pictWidth = width;
            AirplaneEntity = new AirplaneEntity(speed, weight, bodyColor);
        }

        protected PaintAirplane(int speed, double weight, Color bodyColor, int width, int height, int planeWidth, int planeHeight)
        {
            if (this.planeWidth > width || this.planeHeight > height) { return; }

            pictHeight = height;
            pictWidth = width;
            this.planeWidth = planeWidth;
            this.planeHeight = planeHeight;
            AirplaneEntity = new AirplaneEntity(speed, weight, bodyColor);
        }

        public void SetPosition(int x, int y)
        {
            if ((x >= 0 && y >= 0)&&(x + planeHeight <= pictHeight || y + planeWidth <= pictWidth)) {
                startPosX = x;
                startPosY = y;
            }
        }

        public void Move(Movement dir)
        {
            if (!CanMove(dir) || AirplaneEntity == null)
            {
                return;
            }
            switch (dir)
            {
                //влево
                case Movement.Left:
                    startPosX -= (int)AirplaneEntity.Step;
                    break;
                //вверх
                case Movement.Up:
                    startPosY -= (int)AirplaneEntity.Step;
                    break;
                // вправо
                case Movement.Right:
                    startPosX += (int)AirplaneEntity.Step;
                    break;
                //вниз
                case Movement.Down:
                    startPosY += (int)AirplaneEntity.Step;
                    break;
            }

        }

        public virtual void DrawTransport(Graphics g)
        {
            if (AirplaneEntity == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush brush = new SolidBrush(AirplaneEntity.BodyColor);

            //основа
            g.FillRectangle(brush, startPosX + planeWidth / 44, startPosY + planeHeight / 2, planeWidth - 2* planeWidth/11, planeHeight / 2 - planeHeight / 8);
            g.FillEllipse(brush, new Rectangle(startPosX, startPosY + planeHeight / 2, planeWidth / 24, planeHeight / 2 - planeHeight / 8));
            g.DrawLine(pen, new Point(startPosX + planeWidth / 44, startPosY + planeHeight / 2), new Point(startPosX + planeWidth - planeWidth / 6, startPosY + planeHeight / 2));
            g.DrawLine(pen, new Point(startPosX + planeWidth / 44, startPosY + planeHeight - planeHeight / 8), new Point(startPosX + planeWidth - planeWidth / 6, startPosY + planeHeight - planeHeight / 8));


            g.DrawArc(pen, new Rectangle(startPosX, startPosY + planeHeight / 2, planeWidth / 24, planeHeight / 2 - planeHeight / 8), 90.0f, 180.0f);

            

            DrawTriangle(g, pen,
                startPosX + planeWidth - planeWidth / 6, startPosY + planeHeight / 2, // 1
                startPosX + planeWidth, startPosY + 5 * planeHeight / 7, // 2
                startPosX + planeWidth - planeWidth / 6, startPosY + planeHeight - planeHeight / 8); // 3

            DrawTriangle(g, pen,
                startPosX, startPosY, // 1
                startPosX + planeWidth / 4, startPosY + planeHeight / 2, // 2
                startPosX, startPosY + planeHeight / 2); // 3
            g.DrawLine(pen, new Point(startPosX + planeWidth - planeWidth / 6, startPosY + 5 * planeHeight / 7),
                new Point(startPosX + planeWidth, startPosY + 5 * planeHeight / 7));
            // колеса
            g.DrawLine(pen, new Point(startPosX + planeWidth / 4, startPosY + planeHeight - planeHeight / 8),
                new Point(startPosX + planeWidth / 4, startPosY + planeHeight - planeHeight / 11));
            g.DrawLine(pen, new Point(startPosX + planeWidth - 2 * planeWidth / 9, startPosY + planeHeight - planeHeight / 8),
                new Point(startPosX + planeWidth - 2 * planeWidth / 9, startPosY + planeHeight - planeHeight / 11));
            g.DrawEllipse(pen, new Rectangle(startPosX + planeWidth / 4 - planeWidth / 23, startPosY + 11 * planeHeight / 12,
                planeHeight / 12, planeHeight / 12));
            g.DrawEllipse(pen, new Rectangle(startPosX + planeWidth / 4 + planeWidth / 72, startPosY + 11 * planeHeight / 12,
                planeHeight / 12, planeHeight / 12));

            g.DrawEllipse(pen, new Rectangle(startPosX + planeWidth - 5 * planeWidth / 21, startPosY + 11 * planeHeight / 12,
               planeHeight / 12, planeHeight / 12));

            FillRoundedRectangle(g, new SolidBrush(Color.Black), new Rectangle(startPosX + 2 * planeWidth / 9, startPosY + planeHeight / 2 + planeHeight / 7, planeWidth / 2, planeHeight / 7), 5);
            FillRoundedRectangle(g, new SolidBrush(Color.Black), new Rectangle(startPosX, startPosY + planeHeight / 2 - planeHeight / 9, planeWidth / 5, planeHeight / 6), 5);
        
        }

        protected void DrawTriangle(Graphics g, Pen pen, int x1, int y1, int x2, int y2, int x3, int y3)
        {
            Point p1 = new Point(x1, y1);
            Point p2 = new Point(x2, y2);
            Point p3 = new Point(x3, y3);
            g.DrawLine(pen, p1, p2);
            g.DrawLine(pen, p2, p3);
            g.DrawLine(pen, p3, p1);
        }

        protected static GraphicsPath RoundedRect(Rectangle bounds, int radius)
        {
            int diameter = radius * 2;
            Size size = new Size(diameter, diameter);
            Rectangle arc = new Rectangle(bounds.Location, size);
            GraphicsPath path = new GraphicsPath();

            if (radius == 0)
            {
                path.AddRectangle(bounds);
                return path;
            }

            // top left arc  
            path.AddArc(arc, 180, 90);

            // top right arc  
            arc.X = bounds.Right - diameter;
            path.AddArc(arc, 270, 90);

            // bottom right arc  
            arc.Y = bounds.Bottom - diameter;
            path.AddArc(arc, 0, 90);

            // bottom left arc 
            arc.X = bounds.Left;
            path.AddArc(arc, 90, 90);

            path.CloseFigure();
            return path;
        }

        protected static void DrawRoundedRectangle(Graphics graphics, Pen pen, Rectangle bounds, int cornerRadius)
        {
            if (graphics == null)
                throw new ArgumentNullException(nameof(graphics));
            if (pen == null)
                throw new ArgumentNullException(nameof(pen));

            using (GraphicsPath path = RoundedRect(bounds, cornerRadius))
            {
                graphics.DrawPath(pen, path);
            }
        }

        protected static void FillRoundedRectangle(Graphics graphics, Brush brush, Rectangle bounds, int cornerRadius)
        {
            if (graphics == null)
                throw new ArgumentNullException(nameof(graphics));
            if (brush == null)
                throw new ArgumentNullException(nameof(brush));

            using (GraphicsPath path = RoundedRect(bounds, cornerRadius))
            {
                graphics.FillPath(brush, path);
            }
        }
    }
}
